# Copyright (c) Facebook, Inc. and its affiliates. (http://www.facebook.com)
# -*- coding: utf-8 -*-

"""
tests.utils.__init__

This module loads the modules under src/mapillary/utils for tests

:copyright: (c) 2021 Facebook
:license: MIT LICENSE
"""

# Exraction testing
from . import test_extract  # noqa: F401

# Filter testing
from . import test_filter  # noqa: F401
